"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getShareAppMenuItem = void 0;
var _discoverUtils = require("@kbn/discover-utils");
var _lodash = require("lodash");
var _public = require("@kbn/kibana-utils-plugin/public");
var _i18n = require("@kbn/i18n");
var _get_sharing_data = require("../../../../../utils/get_sharing_data");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the "Elastic License
 * 2.0", the "GNU Affero General Public License v3.0 only", and the "Server Side
 * Public License v 1"; you may not use this file except in compliance with, at
 * your election, the "Elastic License 2.0", the "GNU Affero General Public
 * License v3.0 only", or the "Server Side Public License, v 1".
 */

const getShareAppMenuItem = ({
  discoverParams,
  services,
  stateContainer
}) => {
  return {
    id: _discoverUtils.AppMenuActionId.share,
    type: _discoverUtils.AppMenuActionType.primary,
    controlProps: {
      label: _i18n.i18n.translate('discover.localMenu.shareTitle', {
        defaultMessage: 'Share'
      }),
      description: _i18n.i18n.translate('discover.localMenu.shareSearchDescription', {
        defaultMessage: 'Share Discover session'
      }),
      iconType: 'share',
      testId: 'shareTopNavButton',
      onClick: async ({
        anchorElement
      }) => {
        const {
          dataView,
          isEsqlMode
        } = discoverParams;
        if (!services.share) {
          return;
        }
        const savedSearch = stateContainer.savedSearchState.getState();
        const searchSourceSharingData = await (0, _get_sharing_data.getSharingData)(savedSearch.searchSource, stateContainer.appState.getState(), services, isEsqlMode);
        const {
          locator,
          notifications
        } = services;
        const appState = stateContainer.appState.getState();
        const {
          timefilter
        } = services.data.query.timefilter;
        const timeRange = timefilter.getTime();
        const refreshInterval = timefilter.getRefreshInterval();
        const filters = services.filterManager.getFilters();

        // Share -> Get links -> Snapshot
        const params = {
          ...(0, _lodash.omit)(appState, 'dataSource'),
          ...(savedSearch.id ? {
            savedSearchId: savedSearch.id
          } : {}),
          ...(dataView !== null && dataView !== void 0 && dataView.isPersisted() ? {
            dataViewId: dataView === null || dataView === void 0 ? void 0 : dataView.id
          } : {
            dataViewSpec: dataView === null || dataView === void 0 ? void 0 : dataView.toMinimalSpec()
          }),
          filters,
          timeRange,
          refreshInterval
        };
        const relativeUrl = locator.getRedirectUrl(params);

        // This logic is duplicated from `relativeToAbsolute` (for bundle size reasons). Ultimately, this should be
        // replaced when https://github.com/elastic/kibana/issues/153323 is implemented.
        const link = document.createElement('a');
        link.setAttribute('href', relativeUrl);
        const shareableUrl = link.href;

        // Share -> Get links -> Saved object
        let shareableUrlForSavedObject = await locator.getUrl({
          savedSearchId: savedSearch.id
        }, {
          absolute: true
        });

        // UrlPanelContent forces a '_g' parameter in the saved object URL:
        // https://github.com/elastic/kibana/blob/a30508153c1467b1968fb94faf1debc5407f61ea/src/plugins/share/public/components/url_panel_content.tsx#L230
        // Since our locator doesn't add the '_g' parameter if it's not needed, UrlPanelContent
        // will interpret it as undefined and add '?_g=' to the URL, which is invalid in Discover,
        // so instead we add an empty object for the '_g' parameter to the URL.
        shareableUrlForSavedObject = (0, _public.setStateToKbnUrl)('_g', {}, undefined, shareableUrlForSavedObject);
        services.share.toggleShareContextMenu({
          anchorElement,
          allowEmbed: false,
          allowShortUrl: !!services.capabilities.discover_v2.createShortUrl,
          shareableUrl,
          shareableUrlForSavedObject,
          shareableUrlLocatorParams: {
            locator,
            params
          },
          objectId: savedSearch.id,
          objectType: 'search',
          objectTypeMeta: {
            title: _i18n.i18n.translate('discover.share.shareModal.title', {
              defaultMessage: 'Share this Discover session'
            })
          },
          sharingData: {
            isTextBased: isEsqlMode,
            locatorParams: [{
              id: locator.id,
              params: isEsqlMode ? {
                ...params,
                timeRange: timefilter.getAbsoluteTime() // Will be used when generating CSV on server. See `filtersFromLocator`.
              } : params
            }],
            ...searchSourceSharingData,
            // CSV reports can be generated without a saved search so we provide a fallback title
            title: savedSearch.title || _i18n.i18n.translate('discover.localMenu.fallbackReportTitle', {
              defaultMessage: 'Untitled Discover session'
            })
          },
          isDirty: !savedSearch.id || stateContainer.appState.hasChanged(),
          showPublicUrlSwitch: _get_sharing_data.showPublicUrlSwitch,
          onClose: () => {
            anchorElement === null || anchorElement === void 0 ? void 0 : anchorElement.focus();
          },
          toasts: notifications.toasts
        });
      }
    }
  };
};
exports.getShareAppMenuItem = getShareAppMenuItem;